""" Django Scaffold Tools

Helps to setup DRF to be ready with few lines of code,
with easy convention configuration of serializers and viewsets.
"""
from .version import VERSION

__version__ = VERSION



from django.db import models


class AbstractAttachment(models.Model):
    FILE_FIELD_NAME = ''

    name = models.CharField(
        verbose_name='名称',
        max_length=255,
        blank=True,
        default='',
    )

    ext_url = models.URLField(
        verbose_name='外部附件链接',
        blank=True,
        null=False,
        default='',
    )

    class Meta:
        verbose_name = '抽象附件'
        abstract = True

    def __str__(self):
        return self.name

    def url(self):
        file_field = getattr(self, self.FILE_FIELD_NAME)
        return file_field.url if file_field else self.ext_url

    def save(self, *args, **kwargs):
        file_field = getattr(self, self.FILE_FIELD_NAME)
        if not self.name and file_field:
            self.name = file_field.name
        super().save(*args, **kwargs)


class Image(AbstractAttachment,
            models.Model):
    """ 图片
    TODO: 考虑支持七牛引擎的问题
    TODO: 还要考虑裁剪缩略图的问题
    TODO: 存放的路径要按照 md5sum 折叠，以节省空间
    """
    FILE_FIELD_NAME = 'image'

    image = models.ImageField(
        verbose_name='图片',
        upload_to='images/',
        null=False,
        blank=True,
    )

    class Meta:
        verbose_name = '图片'
        verbose_name_plural = '图片'
        db_table = 'base_media_image'

    def set_file_path(self, path):
        from django.core.files import File
        import os.path
        self.image.save(os.path.basename(path), File(open(path, 'rb')))
        self.save()

    @classmethod
    def from_file_path(cls, path):
        """ 工厂方法
        根据服务器本地文件路径构造一个对象
        :return:
        """
        from django.core.files import File
        import os.path
        obj = cls()
        obj.image.save(os.path.basename(path), File(open(path, 'rb')))
        obj.save()
        # print(obj, obj.id)
        return obj

    @classmethod
    def from_file(cls, file):
        """ TODO: 工厂方法
        根据提交的 file 构造一个对象
        如果存在相同的 md5sum
        :return:
        """

    @classmethod
    def from_url_reference(cls, url):
        """ 工厂方法
        根据指定的 url 生成一个引用的 Attachment 对象
        :return:
        """
        return cls.objects.create(ext_url=url, name=url.split('/')[-1])

    @classmethod
    def from_url_download(cls, url):
        """ TODO: 工厂方法
        根据指定的 url 下载图片保存，生成一个对象
        :return:
        """
        from django_base.base_utils.app_error.exceptions import AppError
        raise AppError(99999, '尚未实现', debug=True)


class Video(AbstractAttachment,
            models.Model):
    """ 视频对象
    """

    video = models.FileField(
        verbose_name='视频',
        upload_to='video/'
    )

    class Meta:
        verbose_name = '视频'
        verbose_name_plural = '视频'
        db_table = 'base_media_video'


class Audio(AbstractAttachment,
            models.Model):
    """ 音频对象
    """

    audio = models.FileField(
        verbose_name='音频',
        upload_to='audio/'
    )

    is_active = models.BooleanField(
        verbose_name='是否可用',
        default=True,
    )

    class Meta:
        verbose_name = '音频'
        verbose_name_plural = '音频'
        db_table = 'base_media_audio'


class GalleryModel(models.Model):
    images = models.ManyToManyField(
        verbose_name='图片',
        to=Image,
        related_name='%(class)ss_attached',
        blank=True,
    )

    class Meta:
        abstract = True

    @property
    def images_url(self):
        return [img.url() for img in self.images.all()]


class Attachment(AbstractAttachment,
                 models.Model):
    FILE_FIELD_NAME = 'file'

    file = models.FileField(
        verbose_name='上传附件',
        upload_to='attachment/',
        null=False,
        blank=True,
    )

    class Meta:
        verbose_name = '附件'
        verbose_name_plural = '附件'
        db_table = 'base_media_attachment'



"""
Django scaffold tools - https://github.com/easecloud/django-scaffold tools

Authors: Alfred Huang (黄文超) <https://github.com/fish-ball>

2020-now @ Copyright Easecloud

Licensed under the Apache License, Version 2.0 (the 'License');
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an 'AS IS' BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

------

Django all-in-one settings module.

Configured from 'django-admin startproject' using Django 3.0.

For more information on this file, see
https://docs.djangoproject.com/en/3.0/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/3.0/ref/settings/

------

Environment variables:

- DJANGO_SECRET_KEY: the secret_key used by the django settings.py module.
- DJANGO_DEBUG: if you want to enable debug, set it as 1.
- DJANGO_ALLOWED_HOSTS: comma separated allowed host list. default '*'
- DJANGO_LANGUAGE: example (en-us), default zh-hans
- DJANGO_TIME_ZONE: example (UTC), default Asia/Shanghai

- DJANGO_DB_TYPE: sqlite3 / mysql(default), if more type wanted, pull-requests are welcomed.
- DJANGO_DB_HOST: default 127.0.0.1
- DJANGO_DB_PORT: default 3306
- DJANGO_DB_NAME: default django_{app_name}
    (if using sqlite3, it stands for db filename, default db.sqlite3)
- DJANGO_DB_USER: default root
- DJANGO_DB_PASS: default root
- DJANGO_DB_CHARSET: default utf8mb4
- DJANGO_DB_COLLATION: default utf8mb4_general_ci
"""

import os
import sys

from django.core.exceptions import ImproperlyConfigured

# Get the main module name
app_name = os.environ['DJANGO_SETTINGS_MODULE'].split('.')[0]

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
# https://stackoverflow.com/a/606574/2544762
BASE_DIR = os.path.dirname(os.path.abspath(sys.modules['__main__'].__file__))

# Quick-start development base_settings - unsuitable for production
# See https://docs.djangoproject.com/en/3.0/howto/deployment/checklist/

SECRET_KEY = os.environ.get('DJANGO_SECRET_KEY', 'default-secret-key')

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = os.environ.get('DJANGO_DEBUG') == '1'

ALLOWED_HOSTS = os.environ.get('DJANGO_ALLOWED_HOSTS', '*').split(',')

SECURE_PROXY_SSL_HEADER = ('HTTP_X_FORWARDED_PROTO', 'https')
# SECURE_PROXY_SSL_HEADER = ('HTTP_X_REAL_IP', '127.0.0.1')

# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    # Included third-party apps
    # 'scaffold.modules',
    # 'scaffold.models.entity.media',
    'rest_framework',
    'django_cron',
    'django_filters',
]

MIDDLEWARE = [
    # 'django_base.base_utils.middleware.MethodOverrideMiddleware',
    # 'django_base.base_utils.middleware.ExplicitSessionMiddleware',
    # 'django_base.base_utils.middleware.CustomExceptionMiddleware',
    # 'django_base.base_utils.app_error.middleware.AppErrorMiddleware',
    # 'django_base.base_utils.middleware.GlobalRequestMiddleware',
    # 生产环境建议关闭，使用手动定义的 MEDIA_URL 以及 STATIC_URL
    # 'django_base.base_utils.middleware.FullMediaUrlMiddleware',
    # 'django_base.base_utils.middleware.DebugMiddleware',
    # 'django_base.base_utils.middleware.CookieCsrfMiddleware',
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = f'{app_name}.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = f'{app_name}.wsgi.application'

# Database
# https://docs.djangoproject.com/en/3.0/ref/settings/#databases

DATABASES = dict()

_db_type: str = os.environ.get('DJANGO_DB_TYPE', 'mysql').lower()
if _db_type == 'sqlite3':
    _db_name = os.environ.get('DJANGO_DB_NAME', 'db.sqlite3')
    DATABASES['default'] = {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': _db_name if _db_name.startswith('/') else os.path.join(
            BASE_DIR,
            os.environ.get('DJANGO_DB_NAME', 'db.sqlite3')
        ),
    }
elif _db_type == 'mysql':
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.mysql',
            'NAME': os.environ.get('DJANGO_DB_NAME', f'django_{app_name}'),
            'USER': os.environ.get('DJANGO_DB_USER', 'root'),
            'PASSWORD': os.environ.get('DJANGO_DB_USER', 'root'),
            'HOST': os.environ.get('DJANGO_DB_HOST', '127.0.0.1'),
            'PORT': os.environ.get('DJANGO_DB_PORT', '3306'),
            'OPTIONS': {
                'charset': os.environ.get('DJANGO_DB_CHARSET', 'utf8mb4'),
                'init_command': '''
                    SET default_storage_engine=MYISAM;
                    SET sql_mode='STRICT_TRANS_TABLES';
                ''',
            },
            'TEST': {
                'CHARSET': os.environ.get('DJANGO_DB_CHARSET', 'utf8mb4'),
                'COLLATION': os.environ.get('DJANGO_DB_COLLATION', 'utf8mb4_general_ci'),
            },
        },
    }
else:
    raise ImproperlyConfigured(
        f'The DJANGO_DB_TYPE \'{_db_type}\' you\'ve configured in your '
        f'environment is not supported by dajngo-scaffold-tools right now. '
        f'So please set a supported db type in (sqlite3, mysql),'
        f'or setting DATABASES settings by yourself. '
        f'If you want more DB type to be supported, a pull request is welcomed.'
    )

# Password validation
# https://docs.djangoproject.com/en/3.0/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]

# Internationalization
# https://docs.djangoproject.com/en/3.0/topics/i18n/

LANGUAGE_CODE = os.environ.get('DJANGO_LANGUAGE', 'zh-hans')

TIME_ZONE = os.environ.get('DJANGO_TIME_ZONE', 'Asia/Shanghai')

USE_I18N = True

USE_L10N = True

USE_TZ = False

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/3.0/howto/static-files/

STATIC_ROOT = os.path.join(BASE_DIR, 'static')
STATIC_URL = '/static/'

MEDIA_ROOT = os.path.join(BASE_DIR, 'media')
MEDIA_URL = '/media/'

FILE_UPLOAD_PERMISSIONS = 0o644

# Django Cache
# To solve django-cron process lock issue, we switch the cache to filebased
# https://github.com/Tivix/django-cron/issues/41
CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.filebased.FileBasedCache',
        'LOCATION': '/var/tmp/django_cache',
    }
}

# CORS headers
# https://pypi.org/project/django-cors-headers/

if os.environ.get('DJANGO_CORS') == '1':
    INSTALLED_APPS.insert(
        INSTALLED_APPS.index('django.contrib.staticfiles') + 1,
        'corsheaders'
    )
    MIDDLEWARE.insert(
        MIDDLEWARE.index('django.middleware.common.CommonMiddleware') + 1,
        'corsheaders.middleware.CorsMiddleware',
    )

CORS_ORIGIN_ALLOW_ALL = DEBUG
CORS_ORIGIN_REGEX_WHITELIST = r'.*'
CORS_ALLOW_CREDENTIALS = True
CSRF_TRUSTED_ORIGINS = ['null', '.local', 'localhost', '127.0.0.1']

# REST Framework
# https://www.django-rest-framework.org/

REST_FRAMEWORK = {
    'PAGE_SIZE': 10,
    'DEFAULT_PAGINATION_CLASS':
        'scaffold.restframework.pagination.PageNumberPagination',
    # https://stackoverflow.com/a/30875830/2544762
    'DEFAULT_AUTHENTICATION_CLASSES': [
        'scaffold.restframework.authentication.CsrfExemptSessionAuthentication',
        'rest_framework.authentication.BasicAuthentication',
    ],
    # 'DEFAULT_PERMISSION_CLASSES': [
    #     'rest_framework.permissions.DjangoModelPermissionsOrAnonReadOnly',
    # ],
    'DEFAULT_FILTER_BACKENDS': (
        # 'rest_framework_filters.backends.DjangoFilterBackend',
        # 'rest_framework.filters.SearchFilter',
        # 'rest_framework.filters.OrderingFilter',
        'django_filters.rest_framework.DjangoFilterBackend',
        'scaffold.restframework.filters.DeepFilterBackend',
        'scaffold.restframework.filters.OrderingFilter',
        'scaffold.restframework.filters.SearchFilter',
    ),
    # 'DEFAULT_RENDERER_CLASSES': (
    #     # 'rest_framework.renderers.JSONRenderer',
    #     'rest_framework.renderers.BrowsableAPIRenderer',
    # ),
    'DATE_FORMAT': '%Y-%m-%d',
    'DATETIME_FORMAT': '%Y-%m-%d %H:%M:%S',
    'COERCE_DECIMAL_TO_STRING': False,
    'EXCEPTION_HANDLER': 'core.exceptions.exception_handler',

    # Parser classes priority-wise for Swagger
    'DEFAULT_PARSER_CLASSES': [
        'rest_framework.parsers.FormParser',
        'rest_framework.parsers.MultiPartParser',
        'rest_framework.parsers.JSONParser',
    ],
}

# # Geo
#
# AUTO_GEO_DECODE = False
#
# # 百度地图 API_KEY
# BMAP_KEY = ''
# # 高德地图 API_KEY
# AMAP_KEY = ''
#
# # 是否将音频自动转换为 ogg/mp3
# NORMALIZE_AUDIO = True

# # ============== Payment ===============
#
# # 如果开启调试，所有实际支付的金额会变成 1 分钱
# PAYMENT_DEBUG = True

# # =============== SMS Config ===================
#
# SMS_ACCESS_KEY_ID = '----------'
# SMS_ACCESS_KEY_SECRET = '----------'
#
# SMS_SEND_INTERVAL = 60  # 短信发送时间间隔限制
# SMS_EXPIRE_INTERVAL = 1800
# SMS_SIGN_NAME = '短信签名'
# SMS_TEMPLATES = dict(
#     SIGN_IN='SMS_142655055',
#     CHANGE_PASSWORD='SMS_142655052',
#     CHANGE_MOBILE_VERIFY='SMS_142655051',
#     CHANGE_MOBILE_UPDATE='SMS_142655056',
# )
# SMS_DEBUG = False  # 不真正发送短信，将验证码直接返回
#
# # =============== JPUSH ==================
#
# JPUSH_APP_KEY = 'a286ac61a069e1bb673c122f'
# JPUSH_MASTER_SECRET = 'cf7d0e9a881dc602d23b219d'
#
# # ============== URLS ================
#
# ROOT_URLCONF = 'apps.core.urls'
# ENABLE_ADMIN_SITE = True
#
# # ============== CUSTOM SESSION HEADER ==============
# CUSTOM_SESSION_HEADER = 'SESSION-ID'
#
# # ============== MethodOverrideMiddleware =================
# METHOD_OVERRIDE_ALLOWED_HTTP_METHODS =\
# ['GET', 'HEAD', 'PUT', 'POST', 'DELETE', 'OPTIONS', 'PATCH']
# METHOD_OVERRIDE_PARAM_KEY = '_method'
# METHOD_OVERRIDE_HTTP_HEADER = 'HTTP_X_HTTP_METHOD_OVERRIDE'
#
#
# API_DEBUG = False
#
# REST_DEEP_DEFAULT_DISTINCT = False



from rest_framework import serializers
from . import models as m


class AttachmentSerializer(serializers.ModelSerializer):
    url = serializers.URLField(read_only=True)

    class Meta:
        model = m.Attachment
        fields = '__all__'


class ImageSerializer(serializers.ModelSerializer):
    url = serializers.URLField(read_only=True)

    class Meta:
        model = m.Image
        fields = '__all__'



# Generated by Django 3.0.8 on 2020-08-03 21:21

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Attachment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, default='', max_length=255, verbose_name='名称')),
                ('ext_url', models.URLField(blank=True, default='', verbose_name='外部附件链接')),
                ('file', models.FileField(blank=True, upload_to='attachment/', verbose_name='上传附件')),
            ],
            options={
                'verbose_name': '附件',
                'verbose_name_plural': '附件',
                'db_table': 'base_media_attachment',
            },
        ),
        migrations.CreateModel(
            name='Audio',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, default='', max_length=255, verbose_name='名称')),
                ('ext_url', models.URLField(blank=True, default='', verbose_name='外部附件链接')),
                ('audio', models.FileField(upload_to='audio/', verbose_name='音频')),
                ('is_active', models.BooleanField(default=True, verbose_name='是否可用')),
            ],
            options={
                'verbose_name': '音频',
                'verbose_name_plural': '音频',
                'db_table': 'base_media_audio',
            },
        ),
        migrations.CreateModel(
            name='Image',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, default='', max_length=255, verbose_name='名称')),
                ('ext_url', models.URLField(blank=True, default='', verbose_name='外部附件链接')),
                ('image', models.ImageField(blank=True, upload_to='images/', verbose_name='图片')),
            ],
            options={
                'verbose_name': '图片',
                'verbose_name_plural': '图片',
                'db_table': 'base_media_image',
            },
        ),
        migrations.CreateModel(
            name='Video',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, default='', max_length=255, verbose_name='名称')),
                ('ext_url', models.URLField(blank=True, default='', verbose_name='外部附件链接')),
                ('video', models.FileField(upload_to='video/', verbose_name='视频')),
            ],
            options={
                'verbose_name': '视频',
                'verbose_name_plural': '视频',
                'db_table': 'base_media_video',
            },
        ),
    ]



from .meta import *


class AbstractMember(EntityModel):
    """ 会员类
    从基础的用户角色扩展，
    """

    GENDER_SECRET = ''
    GENDER_MALE = 'M'
    GENDER_FEMALE = 'F'
    GENDER_CHOICES = (
        (GENDER_SECRET, '保密'),
        (GENDER_MALE, '男'),
        (GENDER_FEMALE, '女'),
    )

    user = models.OneToOneField(
        primary_key=True,
        verbose_name='用户',
        to=User,
        related_name='%(class)s',
        db_column='user_id',
        on_delete=models.CASCADE,
    )

    nickname = models.CharField(
        verbose_name='昵称',
        max_length=255,
        blank=True,
        default='',
    )

    gender = models.CharField(
        verbose_name='性别',
        max_length=1,
        choices=GENDER_CHOICES,
        default=GENDER_SECRET,
        blank=True,
    )

    real_name = models.CharField(
        verbose_name='真实姓名',
        max_length=150,
        blank=True,
        default='',
    )

    mobile = models.CharField(
        verbose_name='手机号码',
        max_length=45,
        unique=True,
        null=True,
        blank=True,
        # base_validator=base_validator.validate_mobile,
    )

    birthday = models.DateField(
        verbose_name='生日',
        null=True,
        blank=True,
    )

    age = models.IntegerField(
        verbose_name='年龄',
        default=0,
        help_text='如果为0则根据birthday判定年龄，否则使用此数字作为年龄'
    )

    avatar = models.OneToOneField(
        verbose_name='头像',
        to='media.Image',
        related_name='%(class)s',
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
    )

    search_history = models.TextField(
        verbose_name='搜索历史',
        null=True,
        blank=True,
        help_text='最近10次搜索历史，逗号分隔'
    )

    signature = models.TextField(
        verbose_name='个性签名',
        null=True,
        blank=True,
    )

    district = models.IntegerField(
        verbose_name='所在地区',
        help_text='行政区划编码，参考：https://zh.wikipedia.org/wiki/中华人民共和国行政区划',
        null=False,
        default=0,
    )

    address = models.TextField(
        verbose_name='详细地址',
        blank=True,
        default='',
    )

    session_key = models.CharField(
        verbose_name='session_key',
        max_length=255,
        blank=True,
        default='',
        help_text='用于区分单用例登录',
    )

    class Meta:
        abstract = True

    def __str__(self):
        return '{}:{}'.format(self.mobile, self.nickname)

    def save(self, *args, **kwargs):
        super().save(*args, **kwargs)
        # 将用户名和 is_active 同步到 User
        # 更换绑定手机要用到
        # self.user.username = self.mobile
        # self.user.is_active = self.is_active
        # self.user.save()

    def delete(self, *args, **kwargs):
        """ 删除的时候要连带删除 User 对象
        :param args:
        :param kwargs:
        :return:
        """
        self.user.delete()
        return super().delete(*args, **kwargs)

    # def get_age(self):
    #     import time
    #     today = time.gmtime()
    #     dy = 0
    #     if self.age:
    #         return self.age
    #     if self.birthday:
    #         birthday = self.birthday
    #         dd = today[2] - birthday.day
    #         dm = today[1] - birthday.month
    #         dy = today[0] - birthday.year
    #         if dd < 0:
    #             dd = dd + 30
    #             dm = dm - 1
    #             if dm < 0:
    #                 dm = dm + 12
    #                 dy = dy - 1
    #         if dm < 0:
    #             dm = dm + 12
    #             dy = dy - 1
    #         return dy.__str__()
    #     return None


# class MemberAddress(UserOwnedModel,
#                     EntityModel):
#     """ 会员地址，可以用于收货地址等用途
#     """
#     district = models.CharField(
#         verbose_name='地区',
#         max_length=20,
#         help_text='行政区划编号',
#     )
#
#     content = models.CharField(
#         verbose_name='详细地址',
#         max_length=255,
#     )
#
#     receiver = models.CharField(
#         verbose_name='收件人',
#         max_length=50,
#     )
#
#     mobile = models.CharField(
#         verbose_name='联系电话',
#         max_length=20,
#     )
#
#     is_default = models.BooleanField(
#         verbose_name='是否默认',
#         default=False,
#     )
#
#     class Meta:
#         verbose_name = '地址'
#         verbose_name_plural = '地址'
#         db_table = 'member_address'

class MemberPinyinMixin(models.Model):
    nickname_pinyin = models.CharField(
        verbose_name='昵称拼音',
        max_length=255,
        blank=True,
        default='',
    )

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        # 生成昵称的拼音
        from uuslug import slugify
        self.nickname_pinyin = slugify(self.nickname)
        super().save(*args, **kwargs)


class MemberConstellationMixin(models.Model):
    CONSTELLATION_ARIES = 'ARIES'
    CONSTELLATION_TAURUS = 'TAURUS'
    CONSTELLATION_GEMINI = 'GEMINI'
    CONSTELLATION_CANCER = 'CANCER'
    CONSTELLATION_LEO = 'LEO'
    CONSTELLATION_VIRGO = 'VIRGO'
    CONSTELLATION_LIBRA = 'LIBRA'
    CONSTELLATION_SCORPIO = 'SCORPIO'
    CONSTELLATION_SAGITTARIUS = 'SAGITTARIUS'
    CONSTELLATION_CAPRICORN = 'CAPRICORN'
    CONSTELLATION_AQUARIUS = 'AQUARIUS'
    CONSTELLATION_PISCES = 'PISCES'

    CONSTELLATION_CHOICES = (
        (CONSTELLATION_ARIES, '白羊座'),
        (CONSTELLATION_TAURUS, '金牛座'),
        (CONSTELLATION_GEMINI, '双子座'),
        (CONSTELLATION_CANCER, '巨蟹座'),
        (CONSTELLATION_LEO, '狮子座'),
        (CONSTELLATION_VIRGO, '处女座'),
        (CONSTELLATION_LIBRA, '天秤座'),
        (CONSTELLATION_SCORPIO, '天蝎座'),
        (CONSTELLATION_SAGITTARIUS, '射手座'),
        (CONSTELLATION_CAPRICORN, '摩羯座'),
        (CONSTELLATION_AQUARIUS, '水瓶座'),
        (CONSTELLATION_PISCES, '双鱼座'),
    )

    constellation = models.CharField(
        verbose_name='星座',
        max_length=45,
        choices=CONSTELLATION_CHOICES,
        blank=True,
        default='',
    )

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        # 如果输入了生日日期，直接确定星座
        if not self.birthday:
            self.birthday = datetime.now()
        if self.birthday:
            date_str = self.birthday.strftime('%m%d')
            if date_str < '0120':  # 摩羯座
                self.constellation = self.CONSTELLATION_CAPRICORN
            elif date_str < '0219':  # 水瓶座
                self.constellation = self.CONSTELLATION_AQUARIUS
            elif date_str < '0321':  # 双鱼座
                self.constellation = self.CONSTELLATION_PISCES
            elif date_str < '0420':  # 白羊座
                self.constellation = self.CONSTELLATION_ARIES
            elif date_str < '0521':  # 金牛座
                self.constellation = self.CONSTELLATION_TAURUS
            elif date_str < '0622':  # 双子座
                self.constellation = self.CONSTELLATION_GEMINI
            elif date_str < '0723':  # 巨蟹座
                self.constellation = self.CONSTELLATION_CANCER
            elif date_str < '0823':  # 狮子座
                self.constellation = self.CONSTELLATION_LEO
            elif date_str < '0923':  # 处女座
                self.constellation = self.CONSTELLATION_VIRGO
            elif date_str < '1024':  # 天秤座
                self.constellation = self.CONSTELLATION_LIBRA
            elif date_str < '1123':  # 天蝎座
                self.constellation = self.CONSTELLATION_SCORPIO
            elif date_str < '1222':  # 射手座
                self.constellation = self.CONSTELLATION_SAGITTARIUS
            else:  # 摩羯座
                self.constellation = self.CONSTELLATION_CAPRICORN
        super().save(*args, **kwargs)


class AbstractOAuthEntry(NullableUserOwnedModel):
    PLATFORM_WECHAT_APP = 'WECHAT_APP'
    PLATFORM_WECHAT_BIZ = 'WECHAT_BIZ'
    PLATFORM_ALIPAY = 'ALIPAY'
    PLATFORM_QQ = 'QQ'
    PLATFORM_WEIBO = 'WEIBO'
    PLATFORM_CHOICES = (
        (PLATFORM_WECHAT_APP, '微信APP'),
        (PLATFORM_WECHAT_BIZ, '微信公众平台'),
        (PLATFORM_ALIPAY, '支付宝'),
        (PLATFORM_QQ, 'QQ'),
        (PLATFORM_WEIBO, '微博'),
    )

    platform = models.CharField(
        verbose_name='第三方平台',
        max_length=20,
        choices=PLATFORM_CHOICES,
        default='',
        blank=True,
    )

    app = models.CharField(
        verbose_name='app',
        max_length=120,
        blank=True,
        default='',
    )

    openid = models.CharField(
        verbose_name='用户OpenID',
        max_length=128,
    )

    unionid = models.CharField(
        verbose_name='Union ID',
        max_length=50,
        blank=True,
    )

    nickname = models.CharField(
        verbose_name='用户昵称',
        max_length=128,
        blank=True,
        null=True,
    )

    headimgurl = models.URLField(
        verbose_name='用户头像',
        blank=True,
        null=True,
    )

    avatar = models.ImageField(
        verbose_name='头像文件',
        upload_to='oauth/avatar/',
        blank=True,
        null=True,
    )

    params = models.TextField(
        verbose_name='params',
        blank=True,
        default=''
    )

    class Meta:
        abstract = True
        verbose_name = '第三方授权'
        verbose_name_plural = '第三方授权'
        db_table = 'member_oauth_entry'
        unique_together = [['app', 'openid']]



""" Custom specified pagination classes """
from collections import OrderedDict

from rest_framework import pagination
from rest_framework.response import Response


class PageNumberPagination(pagination.PageNumberPagination):
    """ 自定义分页器
    基于 rest_framework 的 PageNumberPagination 基础上修改
    """
    page_size = 10
    page_size_query_param = 'page_size'

    def get_paginated_response(self, data):
        return Response(OrderedDict([
            ('count', self.page.paginator.count),
            ('pages', self.page.paginator.num_pages),
            # ('next', self.get_next_link()),
            # ('previous', self.get_previous_link()),
            ('results', data)
        ]))

    def get_page_size(self, request):
        if self.page_size_query_param:
            try:
                # page_size 传入空字符串，0 或者负数时，返回所有内容（依然保留分页格式）
                if not request.query_params[self.page_size_query_param] or \
                        int(request.query_params[self.page_size_query_param]) <= 0:
                    return 1e100
                # 原始的 rest_framework 实现
                return pagination._positive_int(
                    request.query_params[self.page_size_query_param],
                    strict=True,
                    cutoff=self.max_page_size
                )
            except (KeyError, ValueError):
                pass

        return self.page_size



from rest_framework import viewsets

from . import models as m
from . import serializers as s


class ImageViewSet(viewsets.ModelViewSet):
    queryset = m.Image.objects.all()
    serializer_class = s.ImageSerializer
    filter_fields = '__all__'


class AttachmentViewSet(viewsets.ModelViewSet):
    queryset = m.Attachment.objects.all()
    serializer_class = s.AttachmentSerializer
    filter_fields = '__all__'
    ordering = ['-pk']



from django_base.base_utils.app_error.exceptions import AppError


def validate_params(params, validators):
    for key, validator in validators.items():
        if key not in params:
            raise AppError(40000, '缺少参数', data=dict(key=key, params=params))
        if callable(validator):
            if not validator(params[key]):
                raise AppError(40001, '参数校验失败', data=dict(key=key, params=params))



"""
setup
"""
import os

import semver
import setuptools


def versioning(version: str) -> str:
    """ version to specification
    X.Y.Z -> X.Y.devZ
    """
    sem_ver = semver.parse(version)

    major = sem_ver['major']
    minor = sem_ver['minor']
    patch = str(sem_ver['patch'])

    if minor % 2:
        patch = 'dev' + patch

    fin_ver = '%d.%d.%s' % (
        major,
        minor,
        patch,
    )

    return fin_ver


def get_version() -> str:
    """
    read version from VERSION file
    """
    version = '0.0.0'

    with open(
            os.path.join(
                os.path.dirname(__file__),
                'VERSION'
            )
    ) as version_fh:
        # Get X.Y.Z
        version = version_fh.read().strip()
        # versioning from X.Y.Z to X.Y.devZ
        version = versioning(version)

    return version


def get_install_requires() -> [str]:
    """get install_requires"""
    with open('requirements.txt', 'r') as requirements_fh:
        return requirements_fh.read().splitlines()


setuptools.setup(
    version=get_version(),
    packages=setuptools.find_packages('src'),
    package_dir={'': 'src'},
    install_requires=get_install_requires(),
)



""" Extended rest_framework permissions classes """
from rest_framework.permissions import *


class IsAdminOrIsSelf(BasePermission):
    """ has permission if the current user is the owner of the object or admin user """
    def has_object_permission(self, request, view, obj):
        return obj.author == request.user or \
               request.user.is_staff or \
               request.user.is_superuser


class IsAdminOrReadOnly(BasePermission):
    """ has permission only if the current user is admin """
    def has_permission(self, request, view):
        return request.method in SAFE_METHODS or \
               request.user and (request.user.is_staff or request.user.is_superuser)


